@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.2   2006/06/02
+ * Version: 1.2   2006/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -15,6 +15,10 @@
 #ifndef _LINUX_CCS_PROC_H
 #define _LINUX_CCS_PROC_H
 
+#ifndef __user
+#define __user
+#endif
+
 /*************************  Indexes for /proc interfaces.  *************************/
 
 #define CCS_POLICY_DOMAINPOLICY          0
