@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1   2006/04/01
+ * Version: 1.2   2006/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -41,9 +41,20 @@ int SAKURA_MayMount(struct nameidata *nd)
 						break;
 					}
 					if (dentry == vfsmnt->mnt_root || IS_ROOT(dentry)) {
-						if (vfsmnt->mnt_parent == vfsmnt) break;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+						spin_lock(&vfsmount_lock);
+#endif
+						if (vfsmnt->mnt_parent == vfsmnt) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+							spin_unlock(&vfsmount_lock);
+#endif
+							break;
+						}
 						dentry = vfsmnt->mnt_mountpoint;
 						vfsmnt = vfsmnt->mnt_parent;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+						spin_unlock(&vfsmount_lock);
+#endif
 						continue;
 					}
 					dentry = dentry->d_parent;
@@ -54,14 +65,19 @@ int SAKURA_MayMount(struct nameidata *nd)
 		}
 	}
 	if (flag) {
+		int error = -EPERM;
 		const int is_enforce = CheckCCSEnforce(CCS_SAKURA_DENY_CONCEAL_MOUNT);
-		char *page = kmalloc(PAGE_SIZE, GFP_KERNEL);
+		char *page = ccs_alloc(PAGE_SIZE);
 		if (page) {
-			memset(page, 0, PAGE_SIZE);
-			if (realpath_from_dentry(nd->dentry, nd->mnt, page, PAGE_SIZE - 1) == 0) printk("SAKURA-%s: mount %s : Permission denied.\n", GetMSG(is_enforce), page);
-			kfree(page);
+			if (realpath_from_dentry(nd->dentry, nd->mnt, page, PAGE_SIZE - 1) == 0) {
+				const char *exename = GetEXE();
+				printk("SAKURA-%s: mount %s (pid=%d:exe=%s): Permission denied.\n", GetMSG(is_enforce), page, current->pid, exename);
+				if (is_enforce && CheckSupervisor("# %s is requesting\nmount on %s\n", exename, page) == 0) error = 0;
+				ccs_free(exename);
+			}
+			ccs_free(page);
 		}
-		if (is_enforce) return -EPERM;
+		if (is_enforce) return error;
 	}
 	return 0;
 }
